/*

   remember v2.10

   10/29/1996

   "X.c"      defining functions and variables concerning X
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <string.h>
#include <stdio.h>

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Xaw/Form.h>
#include <X11/Shell.h>
#include <X11/Xaw/Cardinals.h>

#include "kanji.h"
#include "nap.h"

#include "remember.h"
#include "X.h"

#include "Remember_Logo.xbm"

XtAppContext app_con;
Widget toplevel;
XtIntervalId Timer1, Timer2;

Display *disp;
Pixmap pmap, logoPmap;
GC gc, engGC, japGC;
Font engFont;
XEvent ev;
int screen;

static int x, y, mX, mY;
static int wW, wH, oW = 0, oH = 0;
static int ejX, engY, japY;
static Bool mapped = False;

String fallback_resources[] = {
  "Remember*width: 360",
  "Remember*height: 80",
  "*translations: #override \\n\
  <Configure>: configureEvent()\\n\
  <Expose>: exposeEvent()\\n\
  <BtnDown>: buttonPressEvent()\\n\
  <BtnUp>: buttonReleaseEvent()\\n\
  <Key>q: Quit()",
  NULL,
};

XtActionsRec actions[] = {
  {"buttonPressEvent", buttonPressEvent},
  {"buttonReleaseEvent", buttonReleaseEvent},
  {"configureEvent", configureEvent},
  {"exposeEvent", exposeEvent},
  {"Quit", Quit},
};

void realizeWindow(){

  XtRealizeWidget(toplevel);

}

void widgetInit(int *argc, char **argv){

  toplevel = XtVaAppInitialize(&app_con, "Remember", NULL, ZERO, 
			       argc, argv, fallback_resources, NULL);
  XtAppAddActions(app_con, actions, XtNumber(actions));

}

void xInit(){

  XColor gray, def;

  disp = XtDisplay(toplevel);

  screen = DefaultScreen(disp);

  XAllocNamedColor(disp, DefaultColormap(disp, 0),
		   "gray84", &gray, &def);

  logoPmap = XCreatePixmapFromBitmapData(disp, RootWindow(disp, 0),
				   Remember_Logo_bits,
				   Remember_Logo_width, 
				   Remember_Logo_height,
				   gray.pixel,
				   WhitePixel(disp, screen),
				   DefaultDepth(disp, screen));

  engFont = XLoadFont(disp, ENG_FONT_NAME);
  gc = XCreateGC(disp, RootWindow(disp, screen), 0, NULL);
  XSetForeground(disp, gc, WhitePixel(disp, screen));
  engGC = XCreateGC(disp, RootWindow(disp, 0), 0, NULL);
  japGC = XCreateGC(disp, RootWindow(disp, 0), 0, NULL);
  XSetForeground(disp, engGC, BlackPixel(disp, screen));
  XSetForeground(disp, japGC, BlackPixel(disp, screen));
  XSetFont(disp, engGC, engFont);
  setJFont(disp, japGC, FONT24);

  XSelectInput(disp, RootWindow(disp, screen), PointerMotionMask);


}

void timerSet(){

  Timer1 = XtAppAddTimeOut(app_con, 1000, dispDevice, NULL);
  if (chaseMode == CHASE)
    Timer2 = XtAppAddTimeOut(app_con, 1000, chase, NULL);
}

void windowEventLoop(){

  timerSet();
  arrangePTS();

  while(1){
    nap(10000);
    if(XtAppPending(app_con)){
      XtAppNextEvent(app_con, &ev);
      if(!rootMotionEvent(&ev))
	XtDispatchEvent(&ev);
    }
  }
}

void dispDevice(){

  switch(dispState){
  case STATE_NULL_PTS:
  case STATE_ERASE:
    dispQue();
    dispState = STATE_DISP_QUE;
    Timer1 = XtAppAddTimeOut(app_con,
			     (reverseFlag == True) ? 100 : paceQ * 1000,
			     dispDevice,
			     NULL);
    break;
  case STATE_DISP_QUE:
    dispAns();
    dispState = STATE_DISP_ANS;
    Timer1 = XtAppAddTimeOut(app_con,
			     (reverseFlag == True) ? 100 : paceA * 1000,
			     dispDevice,
			     NULL);
    break;
  case STATE_DISP_ANS:
    erase();
    dispState = STATE_ERASE;
    Timer1 = XtAppAddTimeOut(app_con,
			     (reverseFlag == True) ? 100 : EMP_PACE * 1000,
			     dispDevice,
			     NULL);
    dirControler();
    break;
  }
}

void dispQue(){

  if (current == ENG)
    XDrawString(disp, pmap, engGC, ejX, engY, 
		pTS[pTSI].pT[pTI].english,
		strlen(pTS[pTSI].pT[pTI].english));
  else
    putKanji(disp, pmap, japGC, ejX, japY,
	     pTS[pTSI].pT[pTI].japanese);
  XSetWindowBackgroundPixmap(disp, XtWindow(toplevel), pmap);
  XClearWindow(disp, XtWindow(toplevel));

  oldCur = current;
}

void dispAns(pairTable pT){

  if (oldCur == JAP)
    XDrawString(disp, pmap, engGC, ejX, engY, 
		pTS[pTSI].pT[pTI].english,
		strlen(pTS[pTSI].pT[pTI].english));
  else	
    putKanji(disp, pmap, japGC, ejX, japY, pTS[pTSI].pT[pTI].japanese);

  XSetWindowBackgroundPixmap(disp, XtWindow(toplevel), pmap);
  XClearWindow(disp, XtWindow(toplevel));

}

void erase(){

  clearWin();
  
}

void clearWin(){

  XFillRectangle(disp, pmap, gc, 0, 0, wW, wH);
  XCopyArea(disp, logoPmap, pmap, gc, 0, 0,
	    Remember_Logo_width,
	    Remember_Logo_height,
	    ((wW - Remember_Logo_width) > 0 ? wW - Remember_Logo_width : 0),
	    ((wH - Remember_Logo_height) > 0 ? wH - Remember_Logo_height : 0));


  XSetWindowBackgroundPixmap(disp, XtWindow(toplevel), pmap);
  XClearWindow(disp, XtWindow(toplevel));

}

void chase(){

  if (reverseFlag == False){
    XRaiseWindow(disp, XtWindow(toplevel));
    XMoveWindow(disp, XtWindow(toplevel), mX + 40, mY + 40);
    getWindowGeometry();
  }

  Timer2 = XtAppAddTimeOut(app_con, CHASE_TIMER, chase, NULL);
}

void waitForMapping(){

  while(1)
    if(XtAppPending(app_con) != 0){ 
      XtAppNextEvent(app_con, &ev);
      XtDispatchEvent(&ev);
      if(mapped)
	break;
    }
}

Bool rootMotionEvent(XEvent *ev){

  if (ev->type == MotionNotify){
    mX = ev->xbutton.x_root;
    mY = ev->xbutton.y_root;
    return True;
  }
  return False;

}

void exposeEvent(){

  if (mapped)
    XCopyArea(disp, pmap, XtWindow(toplevel), gc, 0, 0, wW, wH, 0, 0);

}

void buttonPressEvent(Widget w, XEvent *ev){

  switch(ev->xbutton.button){
  case Button1:
    button1PressEvent();
    break;
  case Button2:
    button2PressEvent();
    break;
  case Button3:
    button3PressEvent();
    break;
  }

}

void button1PressEvent(){

  current = (current == ENG) ? JAP : ENG;

}  

void button2PressEvent(){

  chaseMode = (chaseMode == DONT_CHASE) ? CHASE : DONT_CHASE;
  switch(chaseMode){
  case CHASE:
    Timer2 = XtAppAddTimeOut(app_con, CHASE_TIMER, chase, NULL);
    break;
  case DONT_CHASE:
    XtRemoveTimeOut(Timer2);
    break;
  }
}

void button3PressEvent(){

  reverseFlag = True;
  XtRemoveTimeOut(Timer1);
  Timer1 = XtAppAddTimeOut(app_con, 10, dispDevice, NULL);
  
}

void buttonReleaseEvent(Widget w, XEvent *ev){

  switch(ev->xbutton.button){
  case Button3:
    button3ReleaseEvent();
    break;
  }

}

void button3ReleaseEvent(){

  reverseFlag = False;

}

void configureEvent(){

  getWindowGeometry();
  setWordPosition();
  if((oW != wW || oH != wH) && (oW != 0 && oH != 0))
    XFreePixmap(disp, pmap);		
  if(oW != wW || oH != wH){
    pmap = XCreatePixmap(disp, XtWindow(toplevel), wW, wH, 
			 DefaultDepth(disp, screen));
    mapped = True;
    clearWin();

    if (dispState == STATE_DISP_ANS)
      dispAns(pTS[pTSI].pT[pTI]);
    if ((dispState == STATE_DISP_QUE) || (dispState == STATE_DISP_ANS))
      dispQue(pTS[pTSI].pT[pTI]);
  }
}

void getWindowGeometry(){

  Position px, py;
  Dimension dw, dh;

  oW = wW;      
  oH = wH;
  XtVaGetValues(toplevel, XtNx, &px, NULL);
  XtVaGetValues(toplevel, XtNy, &py, NULL);
  XtVaGetValues(toplevel, XtNwidth, &dw, NULL);
  XtVaGetValues(toplevel, XtNheight, &dh, NULL);
  x = (int)px;   
  y = (int)py;
  wW = (int)dw; 
  wH = (int)dh;

}

void Quit(){
  exit(0);
}

void setWordPosition(){

  ejX = (BASIC_WORD_X_POS * wW) / BASIC_WINDOW_WIDTH;
  engY = (BASIC_ENG_WORD_Y_POS * wH) / BASIC_WINDOW_HEIGHT;
  japY = (BASIC_JAP_WORD_Y_POS * wH) / BASIC_WINDOW_HEIGHT;

}
